@@ -1,11 +1,8 @@
 # -*- coding: utf-8 -*-
 """
-    # shake_auth.auth
+    # Shake.Auth
 
-    
-    --------
     Copyright © 2010–2011 by Lúcuma labs (http://lucumalabs.com).
-
     MIT License. (http://www.opensource.org/licenses/mit-license.php)
 
 """
@@ -26,7 +23,7 @@ from .utils import (bcrypt, LazyUser, Token,
     auth_render, split_hash, hash_sha256, hash_sha512, hash_bcrypt)
 
 
-class UserExistsError(Exception):
+class UserExistsError(ValueError):
     
     def __init__(self, login):
         self.login = login
@@ -35,7 +32,7 @@ class UserExistsError(Exception):
         return "A user with the login '%s' already exists." % self.login
 
 
-class UserDoesNotExistsError(Exception):
+class UserDoesNotExistsError(ValueError):
     
     def __init__(self, login):
         self.login = login
@@ -44,7 +41,7 @@ class UserDoesNotExistsError(Exception):
         return "A user with the login '%s' dosen't exists." % self.login
 
 
-class PasswordTooShortError(Exception):
+class PasswordTooShortError(ValueError):
     
     def __init__(self, minlen):
         self.minlen = minlen
@@ -61,25 +58,22 @@ CREDENTIAL_TOKEN = 'token'
 
 class Auth(object):
     """
-    Parameters:
-        
-        :param settings:
-            .
-        
-        :param db:
-            .
-        
-        :param User:
-            User model.
-        
-        :param app:
-            A shake instance. Can be added later by :method Shake.init_app:
+    # Parameters:
 
+    User
+    :   an User model.
+        
+    app
+    :   a Shake instance. Can be added later by calling `auth.init_app`.
+
+    settings
+    :   named settings. The following settings are available.
     
-    Settings:
+
+    # Settings
     
-    :pepper:
-        Required.
+    pepper
+    :   required.
         A hardcoded (meaning "not stored in the database") system-wide
         secret 'salt'.
         
@@ -90,73 +84,54 @@ class Auth(object):
         **You cannot change this value if there are already stored passwords
         or they all will become invalid**.
     
-    :hash_alg:
-        The name of the hashing algorithm, this can be one of the following:
+    hash_alg
+    :   the name of the hashing algorithm, this can be one of the following:
         'bcrypt', 'sha512' or 'sha256'. If None, we use the best function
         available.
     
-    :hash_cost:
-        Non-negative integer that control the cost of the hash algorithm.
+    hash_cost
+    :   non-negative integer that control the cost of the hash algorithm.
         The number of operations is proportional to 2**cost.
     
-    :sign_in_redirect:
-        .
+    sign_in_redirect
+    :   where to redirect the user, after signing in.
+        This can be a callable.
     
-    :sign_in_url:
-        .
+    sign_out_redirect
+    :   where to redirect the user, after signing out.
+        This can be a callable.
+        Ignored if `sign_out_view` is not None.
+
+    password_minlen
+    :   minimum number of chars the password must have.
+        A `PasswordTooShortError` will be raised if it does not follow
+        this limit.
+
+    reset_expire
+    :   number of hours after the reset-password one time use token become
+        invalid.  This number can be a float to indicate a fraction of an
+        hour (e.g.: 0.5 = 30 minutes).
     
-    :sign_in_view:
-        .
+    sign_in_view
+    :   template used to render the sign in form.
     
-    :sign_out_redirect:
-        .
-    
-    :sign_out_url:
-        .
-    
-    :sign_out_view:
-        .
-    
-    :password_minlen:
-        .
-    
-    :render:
-        .
-    
-    :password_change:
-        Can a user change her password?
-        Default: True.
-    
-    The following settings are used only if `password_change` is True
+    sign_out_view
+    :   template rendered after the user sign out.
         
-        :change_password_url:
-            .
-        
-        :change_password_view:
-            .
+    change_password_view
+    :   template udes to render the "Forgot your password?" form.
     
-    :password_reset:
-        Can a user reset her password?
-        Default: True.
+    reset_password_view
+    :   template to render when resetting a password.
     
-    The following settings are used only if `password_reset` is True
-        
-        :mailer:
-            .
-        
-        :reset_expire:
-            number of hours after the reset-password one time use token become
-            invalid.  This number can be a float to indicate a fraction of an
-            hour (e.g.: 0.5 = 30 minutes).
-        
-        :reset_password_url:
-            .
-        
-        :reset_password_view:
-            .
-        
-        :email_reset_view:
-            .
+    email_reset_view
+    :   template to use to send the reset-password email.
+
+    render
+    :   a render instance (used in all auth controllers)
+
+    mailer
+    :   a mailer instance (used in the reset password controller)
     
     ..:note::
     
@@ -166,34 +141,27 @@ class Auth(object):
         sha_name $ cost $ salt $ salted_and_hashed_password
         - or -
         bcrypt $ cost $ salt salted_and_hashed_password
+
     """
     
     defaults = {
         'pepper': None,
         'hash_alg': None,
         'hash_cost': 12,
-        
-        'sign_in_url': '/sign-in/',
+
         'sign_in_redirect': '/',
-        'sign_in_view': 'auth_sign_in.html',
-        
-        'sign_out_url': '/sign-out/',
         'sign_out_redirect': '/',
-        'sign_out_view': None,
-        
-        'password_minlen': 6,
-        'render': None,
-        
-        'password_change': True,
-        'change_password_url': '/change-password/',
-        'change_password_view': 'auth_change_password.html',
-        
-        'password_reset': True,
-        'mailer': None,
+        'password_minlen': 5,
         'reset_expire': 3,  # hours
-        'reset_password_url': '/reset-password/',
+
+        'sign_in_view': 'auth_sign_in.html',
+        'sign_out_view': None,
         'reset_password_view': 'auth_reset_password.html',
         'email_reset_view': 'auth_email_reset_password.html',
+        'change_password_view': 'auth_change_password.html',
+
+        'render': None,
+        'mailer': None,
     }
 
     LazyUser = LazyUser
@@ -240,15 +208,15 @@ class Auth(object):
         if (hash_alg == 'bcrypt') and (hash_cost < 4):
             settings.hash_cost = 4
         
-        if settings.password_reset:
-            assert settings.reset_expire >= 0, \
-                "`reset_expire` parameter must be a positive number"
-        
+        assert settings.reset_expire >= 0, \
+            "`reset_expire` parameter must be a positive number"
+    
         self.settings = settings
         
         if app:
             self.init_app(app)
     
+    
     def init_app(self, app):
         """This callback can be used to initialize an application for the
         use with this authentication setup.
@@ -259,50 +227,11 @@ class Auth(object):
         def set_user(request):
             self.get_user(request)
     
+
     def get_user(self, request):
         request.__class__.user = self.LazyUser(self.db, self.User)
     
-    def get_urls(self):
-        urls = []
 
-        sign_in_url = self.settings.sign_in_url
-        if sign_in_url.startswith('/'):
-            rule = Rule(sign_in_url, self.sign_in_view,
-                name='auth.sign_in')
-        else:
-            rule = Rule('/sign-in/', redirect_to=sign_in_url,
-                name='auth.sign_in')
-        urls.append(rule)
-        
-        sign_out_url = self.settings.sign_out_url
-        if sign_out_url.startswith('/'):
-            rule = Rule(sign_out_url, self.sign_out_view,
-                name='auth.sign_out')
-        else:
-            rule = Rule('/sign_out/', redirect_to=sign_out_url,
-                name='auth.sign_out')
-        urls.append(rule)
-        
-        if self.settings.password_change:
-            urls.extend([
-                Rule(self.settings.change_password_url,
-                    self.change_password_view, name='auth.change_password'),
-                ])
-        
-        if self.settings.password_reset:
-            assert 'mailshake' in globals().keys(), ('You need the '
-                '`Mailshake` module to use the password reset views.')
-            assert self.settings.mailer, ('You need provide a `mailer` '
-                'parameter to use the password reset views.')
-            
-            urls.extend([
-                Rule(self.settings.reset_password_url,
-                    self.reset_password_view, name='auth.reset_password'),
-                Rule(self.settings.reset_password_url + '<token>/',
-                    self.reset_password_view, name='auth.check_token'),
-                ])
-        return urls
-    
     def authenticate(self, credentials, update=True):
         """Check if there's a matching pair of login/hashed_password in the
         database.
@@ -314,11 +243,11 @@ class Auth(object):
         if login and password:
             return self._loginpass_authentication(login, password, update)
         
-        if self.settings.password_reset:
-            token = credentials.get(CREDENTIAL_TOKEN)
-            if token:
-                return self._token_authentication(token)
+        token = credentials.get(CREDENTIAL_TOKEN)
+        if token:
+            return self._token_authentication(token)
     
+
     def _loginpass_authentication(self, login, password, update):
         # iOS by default makes you write the first letter as uppercase.
         # The username should be case insensitive anyway.
@@ -341,6 +270,7 @@ class Auth(object):
                 user.password = new_password
         return user
     
+
     def _token_authentication(self, token):
         try:
             login, time36, mac = token.split('.')
@@ -364,10 +294,12 @@ class Auth(object):
         
         return user
     
+
     def hash_password(self, password):
         return self.hash_func(password, cost=self.settings.hash_cost,
             pepper=self.settings.pepper)
     
+
     def check_password(self, password, stored_password):
         """Returns a boolean of whether the password is correct.
         
@@ -402,11 +334,13 @@ class Auth(object):
             raise ValueError("Hash algorithm '%s' not available" % hash_alg)
         return stored_password == hashed
     
+
     def set_password(self, user, passw):
         if len(passw) < self.settings.password_minlen:
             raise PasswordTooShortError(self.settings.password_minlen)
         user.password = self.hash_password(passw)
     
+
     def get_reset_token(self, user, time36=None):
         """Make a timestamped one-time-use token that can be used to
         identifying the user.
@@ -433,9 +367,11 @@ class Auth(object):
         token = '%s.%s.%s' % (user.login, time36, mac)
         return Token(token, self.settings.reset_expire)
     
+
     def sign_in(self, request, user):
         self.LazyUser.sign_in(request, user)
     
+
     def sign_out(self, request):
         self.LazyUser.sign_out(request)
     
@@ -474,20 +410,24 @@ class Auth(object):
         self.db.commit()
         return user
     
+
     def render(self, view, **context):
         try:
             return self.settings.render(view, context)
         except TemplateNotFound:
             return auth_render(view, context)
     
+
     def render_to_string(self, view, **context):
         try:
             return self.settings.render.to_string(view, context)
         except TemplateNotFound:
             return auth_render.to_string(view, context)
     
-    def sign_in_view(self, request, **kwargs):
-        """Signs a user in. """
+
+    def sign_in_controller(self, request, **kwargs):
+        """Signs a user in.
+        """
         redirect_to = request.session.get(REDIRECT_FIELD_NAME)
         redirect_to = redirect_to or self.settings.sign_in_redirect
         if callable(redirect_to):
@@ -516,11 +456,14 @@ class Auth(object):
         view = self.settings.sign_in_view
         return self.render(view, **kwargs)
     
-    def sign_out_view(self, request, **kwargs):
+
+    def sign_out_controller(self, request, **kwargs):
         """Signs out a user.
         
-        :param view: Optional full name of a view to display after
+        view
+        :   Optional full name of a view to display after
             signing out the user.
+
         """
         redirect_to = request.session.get(REDIRECT_FIELD_NAME)
         redirect_to = redirect_to or self.settings.sign_out_redirect
@@ -535,10 +478,12 @@ class Auth(object):
             return self.render(view, **kwargs)
         return redirect(redirect_to)
     
-    def reset_password_view(self, request, token=None, **kwargs):
-        """A view to generate a link to reset your password.
+
+    def change_password_controller(self, request, token=None, **kwargs):
+        """A apge to generate a link to reset your password.
         
-        :param token: timestamped one-time-use token.
+        token
+        :   optional timestamped one-time-use token.
         
         """
         kwargs['auth'] = self
@@ -557,7 +502,7 @@ class Auth(object):
                 if request.user and (request.user != user):
                     self.sign_out(request)
                 self.sign_in(request, user)
-                return self.change_password_view(request, user_requested=False)
+                return self.reset_password_controller(request, user_requested=False)
             
             kwargs['error'] = 'WRONG TOKEN'
         
@@ -580,6 +525,7 @@ class Auth(object):
         view = self.settings.reset_password_view
         return self.render(view, **kwargs)
     
+
     def email_token(self, user, token):
         view = self.settings.email_reset_view
         self.render_to_string(view, {'user': user, 'token': token})
@@ -599,7 +545,8 @@ class Auth(object):
 
         mailer.send(email_msg)
     
-    def change_password_view(self, request, user_requested=True, **kwargs):
+
+    def reset_password_controller(self, request, user_requested=True, **kwargs):
         """A view to change your password.
         """
         if not is_valid(request):
@@ -643,3 +590,4 @@ class Auth(object):
         
         view = self.settings.change_password_view
         return self.render(view, **kwargs)
+
